@@ -4,7 +4,6 @@ File Writer for scenarios to commonroad xml-format
 import datetime
 import enum
 import pathlib
-import io
 import os
 from typing import Union, List, Set
 import numpy as np
@@ -19,18 +18,18 @@ from commonroad.geometry.shape import Rectangle, Circle, Polygon, ShapeGroup
 from commonroad.planning.planning_problem import PlanningProblemSet, PlanningProblem
 from commonroad.prediction.prediction import SetBasedPrediction, TrajectoryPrediction
 from commonroad.scenario.intersection import Intersection
-from commonroad.scenario.lanelet import Lanelet, LineMarking, StopLine
+from commonroad.scenario.lanelet import Lanelet, LineMarking, StopLine, LaneletType
 from commonroad.scenario.obstacle import ObstacleRole, ObstacleType, DynamicObstacle, StaticObstacle, Obstacle, \
-    Occupancy, Shape
-from commonroad.scenario.scenario import Scenario, Tag, Location, GeoTransformation
+    Occupancy, Shape, SignalState, EnvironmentObstacle
+from commonroad.scenario.scenario import Scenario, Tag, Location, GeoTransformation, Environment
 from commonroad.scenario.traffic_sign import TrafficSign, TrafficLight, TrafficLightCycleElement
 from commonroad.scenario.trajectory import Trajectory, State
 
 __author__ = "Stefanie Manzinger, Moritz Klischat, Sebastian Maierhofer"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["Priority Program SPP 1835 Cooperative Interacting Automobiles"]
-__version__ = "2020.1"
-__maintainer__ = "Moritz Klischat"
+__version__ = "2020.3"
+__maintainer__ = "Sebastian Maierhofer"
 __email__ = "commonroad-i06@in.tum.de"
 __status__ = "Released"
 
@@ -147,8 +146,8 @@ class CommonRoadFileWriter:
         assert not (source is None and scenario.source is None)
         assert not (tags is None and scenario.tags is None)
 
-        self.scenario = scenario
-        self.planning_problem_set = planning_problem_set
+        self.scenario: Scenario = scenario
+        self.planning_problem_set: PlanningProblemSet = planning_problem_set
         self._root_node = etree.Element('commonRoad')
         self.author = author if author is not None else scenario.author
         self.affiliation = affiliation if affiliation is not None else scenario.affiliation
@@ -228,17 +227,19 @@ class CommonRoadFileWriter:
         self._root_node.set('source', self.source)
 
         try:
-            if self.scenario.benchmark_id:
-                self._root_node.set('benchmarkID', self.scenario.benchmark_id)
+            if self.scenario.scenario_id:
+                self._root_node.set('benchmarkID', str(self.scenario.scenario_id))
         except:
             self._root_node.set('benchmarkID', '-1')
-            print('Warning: No benchmark id set.')
+            print('Warning: No scenario_id set.')
 
         self._root_node.set('date', datetime.datetime.today().strftime('%Y-%m-%d'))
 
     def _add_all_objects_from_scenario(self):
         if self.location is not None:
             self._root_node.append(LocationXMLNode.create_node(self.location))
+        else:
+            self._root_node.append(LocationXMLNode.create_node(Location()))
         self._root_node.append(TagXMLNode.create_node(self.tags))
         for l in self.scenario.lanelet_network.lanelets:
             self._root_node.append(LaneletXMLNode.create_node(l))
@@ -250,6 +251,8 @@ class CommonRoadFileWriter:
             self._root_node.append(IntersectionXMLNode.create_node(intersection))
         for o in self.scenario.obstacles:
             self._root_node.append(ObstacleXMLNode.create_node(o))
+        for b in self.scenario.environment_obstacle:
+            self._root_node.append(EnvironmentObstacleXMLNode.create_node(b))
 
     def _add_all_planning_problems_from_planning_problem_set(self):
         for (
@@ -259,12 +262,10 @@ class CommonRoadFileWriter:
 
     def _dump(self):
         rough_string = etree.tostring(
-            self._root_node, pretty_print=True, encoding='unicode'
+            self._root_node, pretty_print=True, encoding='UTF-8'
         )
-        rough_string = '<?xml version=\"1.0\" encoding=\"utf-8\"?>\n' + rough_string
+        rough_string = rough_string
         return rough_string
-        # reparsed = minidom.parseString(rough_string)
-        # return reparsed.toprettyxml(indent='  ')
 
     def write_to_file(
         self,
@@ -281,7 +282,7 @@ class CommonRoadFileWriter:
         :return:
         """
         if filename is None:
-            filename = self.scenario.benchmark_id
+            filename = str(self.scenario.scenario_id)
 
         if pathlib.Path(filename).is_file():
             if overwrite_existing_file is OverwriteExistingFile.ASK_USER_INPUT:
@@ -301,19 +302,20 @@ class CommonRoadFileWriter:
             else:
                 print('Replace file {}'.format(filename))
 
-        file = open(filename, 'w')
         self._write_header()
         self._add_all_objects_from_scenario()
         self._add_all_planning_problems_from_planning_problem_set()
         if check_validity:
+            # validate xml format 
             self.check_validity_of_commonroad_file(self._dump())
-        file.write(self._dump())
-        file.close()
+
+        tree = etree.ElementTree(self._root_node)
+        tree.write(filename, pretty_print=True, xml_declaration=True, encoding="utf-8")
 
     def write_scenario_to_file(
         self,
         filename: Union[str, None] = None,
-        overwrite_existing_file: OverwriteExistingFile = OverwriteExistingFile.ASK_USER_INPUT,
+        overwrite_existing_file: OverwriteExistingFile = OverwriteExistingFile.ASK_USER_INPUT
     ):
         """
         Write a scenario without planning-problem. If file already exists, it will be overwritten of skipped.
@@ -323,7 +325,7 @@ class CommonRoadFileWriter:
         :return: None
         """
         if filename is None:
-            filename = self.scenario.benchmark_id
+            filename = str(self.scenario.scenario_id)
 
         if pathlib.Path(filename).is_file():
             if overwrite_existing_file is OverwriteExistingFile.ASK_USER_INPUT:
@@ -347,44 +349,11 @@ class CommonRoadFileWriter:
             else:
                 print('Replace file {}'.format(filename))
 
-        with open(filename, 'w') as file_out:
-            self._write_header()
-            self._add_all_objects_from_scenario()
-            file_out.write(self._dump())
-
-    def write_scenario_to_file_io(self, file_io: io.IOBase):
-        """Write a scenario without planning-problem to file_io.
-
-        Args:
-          file_io: File to write to.
-
-        """
         self._write_header()
         self._add_all_objects_from_scenario()
-        self._write_xml_output_to_file(file_io)
 
-    def write_to_file_io(self, file_io: io.IOBase):
-        """Write a scenario including planning-problem to file_io.
-
-        Args:
-          file_io: File to write to.
-
-        """
-        self._write_header()
-        self._add_all_objects_from_scenario()
-        self._add_all_planning_problems_from_planning_problem_set()
-        self._write_xml_output_to_file(file_io)
-
-    def _write_xml_output_to_file(self, file_io: io.IOBase):
-        """Write the dump from self._dump() to file_io.
-
-        Args:
-          file_io: File to write to.
-
-        """
-        output_str = self._dump()
-        CommonRoadFileWriter.check_validity_of_commonroad_file(output_str)
-        file_io.write(output_str)
+        tree = etree.ElementTree(self._root_node)
+        tree.write(filename, pretty_print=True, xml_declaration=True, encoding="utf-8")
 
     @staticmethod
     def check_validity_of_commonroad_file(commonroad_str: str):
@@ -432,6 +401,8 @@ class LocationXMLNode:
         location_node.append(gps_longitude_node)
         if location.geo_transformation is not None:
             location_node.append(GeoTransformationXMLNode.create_node(location.geo_transformation))
+        if location.environment is not None:
+            location_node.append(EnvironmentXMLNode.create_node(location.environment))
 
         return location_node
 
@@ -466,6 +437,34 @@ class GeoTransformationXMLNode:
         return geotransform_node
 
 
+class EnvironmentXMLNode:
+    @classmethod
+    def create_node(cls, environment: Environment) -> etree.Element:
+        """
+        Create XML-Node for a environment
+        :param environment: Environment object
+        :return: node
+        """
+        environment_node = etree.Element('environment')
+        time_node = etree.Element('time')
+        if environment.time.hours < 10:
+            time_node.text = "0" + str(environment.time.hours) + ":" + str(environment.time.minutes) + ":00"
+        else:
+            time_node.text = str(environment.time.hours) + ":" + str(environment.time.minutes)
+        environment_node.append(time_node)
+        time_of_day_node = etree.Element('timeOfDay')
+        time_of_day_node.text = environment.time_of_day.value
+        environment_node.append(time_of_day_node)
+        weather_node = etree.Element('weather')
+        weather_node.text = environment.weather.value
+        environment_node.append(weather_node)
+        underground_node = etree.Element('underground')
+        underground_node.text = environment.underground.value
+        environment_node.append(underground_node)
+
+        return environment_node
+
+
 class TagXMLNode:
     @classmethod
     def create_node(cls, tags: Set[Tag]) -> etree.Element:
@@ -554,14 +553,17 @@ class LaneletXMLNode:
             stop_line_node = LaneletStopLineXMLNode.create_node(lanelet.stop_line)
             lanelet_node.append(stop_line_node)
 
-        if lanelet.lanelet_type is not None:
+        if len(lanelet.lanelet_type) > 0:
             for lanelet_type_element in lanelet.lanelet_type:
                 lanelet_type_node = etree.Element('laneletType')
                 lanelet_type_node.text = str(lanelet_type_element.value)
                 lanelet_node.append(lanelet_type_node)
         else:
-            warnings.warn('<CommonRoadFileWriter/lanelet.lanelet_type> Lanelet %s has not '
-                          'lanelet type' % lanelet.lanelet_id)
+            warnings.warn('<CommonRoadFileWriter/lanelet.lanelet_type> Lanelet %s has no '
+                          'lanelet type! Default lanelet type is used!' % lanelet.lanelet_id)
+            lanelet_type_node = etree.Element('laneletType')
+            lanelet_type_node.text = str(LaneletType.UNKNOWN.value)
+            lanelet_node.append(lanelet_type_node)
 
         if lanelet.user_one_way:
             for user_one_way in lanelet.user_one_way:
@@ -590,7 +592,7 @@ class LaneletXMLNode:
 
 class ObstacleXMLNode:
     @classmethod
-    def create_node(cls, obstacle: Obstacle) -> etree.Element:
+    def create_node(cls, obstacle: Union[Obstacle, DynamicObstacle, StaticObstacle]) -> etree.Element:
         """
         Create XML-Node for an Obstacle
         :param obstacle: Obstacle for creating a node
@@ -600,24 +602,16 @@ class ObstacleXMLNode:
             return DynamicObstacleXMLNode.create_node(obstacle)
         elif type(obstacle) == StaticObstacle:
             return StaticObstacleXMLNode.create_node(obstacle)
+        elif type(obstacle == EnvironmentObstacle):
+            return EnvironmentObstacleXMLNode.create_node(obstacle)
         else:
             raise Exception()
 
-    @classmethod
-    def _obstacle_role_enum_to_string(cls, obstacle_role: ObstacleRole):
-        """
-        Create string for obstalce role
-        """
-        if obstacle_role == ObstacleRole.STATIC:
-            return 'static'
-        elif obstacle_role == ObstacleRole.DYNAMIC:
-            return 'dynamic'
-
     @classmethod
     def create_obstacle_node_header(
         cls, obstacle_id: int, obstacle_role: ObstacleRole, obstacle_type: ObstacleType
     ):
-        obstacle_node = etree.Element(cls._obstacle_role_enum_to_string(obstacle_role)+'Obstacle')
+        obstacle_node = etree.Element(obstacle_role.value+'Obstacle')
         obstacle_node.set('id', str(obstacle_id))
         # role_node = etree.Element('role')
         # role_node.text = cls._obstacle_role_enum_to_string(obstacle_role)
@@ -628,6 +622,26 @@ class ObstacleXMLNode:
         return obstacle_node
 
 
+class EnvironmentObstacleXMLNode:
+    @classmethod
+    def create_node(cls, environment_obstacle: EnvironmentObstacle) -> etree.Element:
+        """
+        Create XML-Node for a EnvironmentObstacle
+        :param environment_obstacle: environment_obstacle for creating a node
+        :return: node
+        """
+        node = ObstacleXMLNode.create_obstacle_node_header(
+            environment_obstacle.obstacle_id,
+            environment_obstacle.obstacle_role,
+            environment_obstacle.obstacle_type,
+        )
+        shape_node = etree.Element('shape')
+        shape_node.extend(ShapeXMLNode.create_node(environment_obstacle.obstacle_shape))
+        node.append(shape_node)
+
+        return node
+
+
 class StaticObstacleXMLNode:
     @classmethod
     def create_node(cls, static_obstacle: StaticObstacle) -> etree.Element:
@@ -687,6 +701,16 @@ class DynamicObstacleXMLNode:
         )
         obstacle_node.append(initial_state_node)
 
+        # write initial signal state if one exists
+        if dynamic_obstacle.initial_signal_state is not None:
+            initial_signal_state_node = etree.Element('initialSignalState')
+            SignalStateXMLNode.create_signal_state_node(
+                dynamic_obstacle.initial_signal_state,
+                initial_signal_state_node,
+                time_step=dynamic_obstacle.initial_signal_state.time_step,
+            )
+            obstacle_node.append(initial_signal_state_node)
+
         # write prediction depending on type
         if isinstance(dynamic_obstacle.prediction, SetBasedPrediction):
             obstacle_node.append(
@@ -697,6 +721,10 @@ class DynamicObstacleXMLNode:
                 cls._create_trajectory_node(dynamic_obstacle.prediction.trajectory)
             )
 
+        # write signal series if it exists
+        if dynamic_obstacle.signal_series is not None and len(dynamic_obstacle.signal_series) > 0:
+            obstacle_node.append(cls._create_signal_series_node(dynamic_obstacle.signal_series))
+
         return obstacle_node
 
     @classmethod
@@ -726,6 +754,21 @@ class DynamicObstacleXMLNode:
             occupancy_set_node.append(OccupancyXMLNode.create_node(occupancy))
         return occupancy_set_node
 
+    @classmethod
+    def _create_signal_series_node(cls, signal_series: List[SignalState]) -> etree.Element:
+        """
+        Create XML-Node for a Trajectory
+        :param trajectory: trajectory for creating a node
+        :return: node
+        """
+        series_node = etree.Element('signalSeries')
+        for signal_state in signal_series:
+            signal_state_node = etree.Element('signalState')
+            series_node.append(
+                SignalStateXMLNode.create_signal_state_node(signal_state, signal_state_node, signal_state.time_step)
+            )
+        return series_node
+
 
 class OccupancyXMLNode:
     @classmethod
@@ -880,12 +923,10 @@ class PolygonXMLNode:
 
 class StateXMLNode:
     @classmethod
-    def create_goal_state_node(
-        cls, state: State, goal_lanelet_ids: List[int]
-    ) -> etree.Element:
+    def create_goal_state_node(cls, state: State, goal_lanelet_ids: List[int]) -> etree.Element:
         """
-        Create XML-Node for a polygon
-        :param polygon: polygon for creating a node
+        Create XML-Node for a state
+        :param state: CommonRoad state
         :param goal_lanelet_ids: contains a list of lanelet ids if a goal state's position is specified lanelet id(s)
         :return: node
         """
@@ -1189,13 +1230,13 @@ class IntersectionXMLNode:
 
             intersection_node.append(incoming_node)
 
-        # if intersection.crossings is not None:
-        #     crossing_node = etree.Element('crossing')
-        #     for crossing_lanelet in intersection.crossings:
-        #         crossing_lanelet_node = etree.Element('crossingLanelet')
-        #         crossing_lanelet_node.set('ref', str(crossing_lanelet))
-        #         crossing_node.append(crossing_lanelet_node)
-        #     intersection_node.append(crossing_node)
+        if intersection.crossings is not None and len(intersection.crossings) > 0:
+            crossing_node = etree.Element('crossing')
+            for crossing_lanelet in intersection.crossings:
+                crossing_lanelet_node = etree.Element('crossingLanelet')
+                crossing_lanelet_node.set('ref', str(crossing_lanelet))
+                crossing_node.append(crossing_lanelet_node)
+            intersection_node.append(crossing_node)
 
         return intersection_node
 
@@ -1209,6 +1250,8 @@ class TrafficSignXMLNode:
             element_node = etree.Element('trafficSignElement')
             sign_id_node = etree.Element('trafficSignID')
             sign_id_node.text = str(element.traffic_sign_element_id.value)
+            if str(element.traffic_sign_element_id.value) == '':
+                warnings.warn('<FileWriter>: Invalid traffic sign ID!')
             element_node.append(sign_id_node)
             for value in element.additional_values:
                 value_node = etree.Element('additionalValue')
@@ -1253,7 +1296,7 @@ class TrafficLightXMLNode:
         if traffic_light.position is not None:
             position_node = etree.Element('position')
             position_node.append(Point(traffic_light.position[0],
-                                       traffic_light.position[0]).create_node())
+                                       traffic_light.position[1]).create_node())
             traffic_light_node.append(position_node)
         if traffic_light.active is not None:
             active_node = etree.Element('active')
@@ -1309,14 +1352,56 @@ class LaneletStopLineXMLNode:
             line_marking_node = LineMarkingXMLNode.create_node(stop_line.line_marking)
             stop_line_node.append(line_marking_node)
 
-        if stop_line.traffic_sign_ref:
-            traffic_sign_ref_node = TrafficSignXMLNode.create_ref_node(stop_line.traffic_sign_ref)
-            stop_line_node.append(traffic_sign_ref_node)
+        if stop_line.traffic_sign_ref is not None:
+            for sign in stop_line.traffic_sign_ref:
+                traffic_sign_ref_node = TrafficSignXMLNode.create_ref_node(sign)
+                stop_line_node.append(traffic_sign_ref_node)
 
-        if stop_line.traffic_light_ref:
-            traffic_light_ref_node = TrafficLightXMLNode.create_ref_node(stop_line.traffic_light_ref)
-            stop_line_node.append(traffic_light_ref_node)
+        if stop_line.traffic_light_ref is not None:
+            for light in stop_line.traffic_light_ref:
+                traffic_light_ref_node = TrafficLightXMLNode.create_ref_node(light)
+                stop_line_node.append(traffic_light_ref_node)
 
         return stop_line_node
 
 
+class SignalStateXMLNode:
+    @classmethod
+    def create_signal_state_node(cls, signal_state: SignalState, signal_state_node: etree.Element,
+                          time_step: int) -> etree.Element:
+        """
+        Create XML-Node for a state
+        :param signal_state: value of the signal state
+        :param signal_state_node: node of the overlying state
+        :return: node
+        """
+        time_node = etree.Element('time')
+        time_node.append(create_exact_node_int(time_step))
+        signal_state_node.append(time_node)
+
+        if hasattr(signal_state, 'indicator_left'):
+            indicator_left = etree.Element('indicatorLeft')
+            indicator_left.text = str(signal_state.indicator_left).lower()
+            signal_state_node.append(indicator_left)
+
+        if hasattr(signal_state, 'indicator_right'):
+            indicator_right = etree.Element('indicatorRight')
+            indicator_right.text = str(signal_state.indicator_right).lower()
+            signal_state_node.append(indicator_right)
+
+        if hasattr(signal_state, 'braking_lights'):
+            braking_lights = etree.Element('brakingLights')
+            braking_lights.text = str(signal_state.braking_lights).lower()
+            signal_state_node.append(braking_lights)
+
+        if hasattr(signal_state, 'hazard_warning_lights'):
+            hazard_warning_lights = etree.Element('hazardWarningLights')
+            hazard_warning_lights.text = str(signal_state.braking_lights).lower()
+            signal_state_node.append(hazard_warning_lights)
+
+        if hasattr(signal_state, 'flashing_blue_lights'):
+            flashing_blue_lights = etree.Element('flashingBlueLights')
+            flashing_blue_lights.text = str(signal_state.flashing_blue_lights).lower()
+            signal_state_node.append(flashing_blue_lights)
+
+        return signal_state_node
