@@ -1,8 +1,8 @@
+import math
 import os
 import platform
 import re
 import subprocess
-import warnings
 from xml.dom import minidom
 import numpy as np
 import xml.etree.ElementTree as et
@@ -10,17 +10,40 @@ from enum import Enum, unique
 from typing import List, Tuple, Union
 from datetime import datetime
 
-from commonroad import SUPPORTED_COMMONROAD_VERSIONS
+from commonroad.common.validity import is_real_number, is_positive
+from commonroad.scenario.scenario import ScenarioID
 from commonroad.scenario.trajectory import State, Trajectory
 
 __author__ = "Murat Üste, Christina Miller, Moritz Klischat"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["BMW CAR@TUM"]
-__version__ = "2020.1"
+__version__ = "2020.3"
 __maintainer__ = "Moritz Klischat"
 __email__ = "commonroad-i06@in.tum.de"
 __status__ = "Released"
 
+
+class SolutionException(Exception):
+    """
+    Main exception class for solution related exceptions.
+    """
+    pass
+
+
+class StateTypeException(SolutionException):
+    """
+    Main exception class for StateType related exceptions.
+    """
+    pass
+
+
+class SolutionReaderException(Exception):
+    """
+    Main exception class for solution reader related exceptions.
+    """
+    pass
+
+
 @unique
 class VehicleType(Enum):
     FORD_ESCORT = 1
@@ -105,10 +128,10 @@ class StateType(Enum):
     Input             -> Input type for ST, KS, and MB vehicle models
     PMInput           -> Input type for PM vehicle model.
     """
-    PM = 'pmState'
+    MB = 'mbState'
     ST = 'stState'
     KS = 'ksState'
-    MB = 'mbState'
+    PM = 'pmState'
     Input = 'input'
     PMInput = 'pmInput'
 
@@ -117,7 +140,7 @@ class StateType(Enum):
         """
         Returns the state fields for the state type.
 
-        :return: List[str] - State fields as list
+        :return: State fields as list
         """
         return StateFields[self.name].value
 
@@ -126,25 +149,52 @@ class StateType(Enum):
         """
         Returns the xml state fields for the state type.
 
-        :return: List[str] - XML names of the state fields as list
+        :return: XML names of the state fields as list
         """
         return XMLStateFields[self.name].value
 
     @classmethod
-    def get_state_type(cls, state: State) -> 'StateType':
+    def get_state_type(cls, state: State, desired_vehicle_model: VehicleModel = None) -> 'StateType':
         """
         Returns the corresponding StateType for the given State object by matching State object's attributes
         to the state fields.
 
-        :param state: State - CommonRoad State object
-        :return: State-Type
+        :param state: CommonRoad State object
+        :param desired_vehicle_model: check if given vehicle_model is supported first
+        :return: corresponding StateType
         """
+        # put desired_vehicle_model first
         attrs = state.attributes
-        for state_fields in StateFields:
-            if not len(attrs) == len(state_fields.value): continue
-            if not all([attr in state_fields.value for attr in attrs]): continue
-            return cls[state_fields.name]
-        raise Exception('Given state is not valid!')
+
+        if desired_vehicle_model is not None:
+            state_fields_all = [StateFields[str(desired_vehicle_model.name)], StateFields.Input, StateFields.PMInput]
+            state_fields_add = []
+            for sf in StateFields:
+                if sf not in state_fields_all: state_fields_add.append(sf)
+
+            state_fields_all += state_fields_add
+
+            for state_fields in state_fields_all:
+                if not len(attrs) >= len(state_fields.value): continue  # >=
+                if not all([sf in attrs for sf in state_fields.value]): continue
+                return cls[state_fields.name]
+        else:
+            state_fields_all = StateFields
+            for state_fields in state_fields_all:
+                if not len(attrs) == len(state_fields.value): continue  # ==
+                if not all([sf in attrs for sf in state_fields.value]): continue
+                return cls[state_fields.name]
+
+        raise StateTypeException('Given state is not valid!')
+
+    @classmethod
+    def check_state_type(cls, vehicle_model: VehicleModel) -> bool:
+        """
+        Checks whether vehicle model can be supported by trajectory.
+        :param vehicle_model: vehicle model enum
+        :return: bool
+        """
+        StateFields(vehicle_model.name)
 
 
 @unique
@@ -156,10 +206,10 @@ class TrajectoryType(Enum):
     Input             -> InputVector type for ST, KS, and MB vehicle models
     PMInput           -> InputVector type for PM vehicle model.
     """
-    PM = 'pmTrajectory'
+    MB = 'mbTrajectory'
     ST = 'stTrajectory'
     KS = 'ksTrajectory'
-    MB = 'mbTrajectory'
+    PM = 'pmTrajectory'
     Input = 'inputVector'
     PMInput = 'pmInputVector'
 
@@ -173,22 +223,24 @@ class TrajectoryType(Enum):
         return StateType[self.name]
 
     @classmethod
-    def get_trajectory_type(cls, trajectory: Trajectory) -> 'TrajectoryType':
+    def get_trajectory_type(cls, trajectory: Trajectory,
+                            desired_vehicle_model: VehicleModel = None) -> 'TrajectoryType':
         """
         Returns the corresponding TrajectoryType for the given Trajectory object based on the StateType of its states.
 
-        :param trajectory: Trajectory - CommonRoad Trajectory object
-        :return: TrajectoryType
+        :param trajectory: CommonRoad Trajectory object
+        :param desired_vehicle_model: check if given vehicle_model is supported first
+        :return: corresponding TrajectoryType
         """
-        state_type = StateType.get_state_type(trajectory.state_list[0])
+        state_type = StateType.get_state_type(trajectory.state_list[0], desired_vehicle_model)
         return cls[state_type.name]
 
     def valid_vehicle_model(self, vehicle_model: VehicleModel) -> bool:
         """
         Checks whether given vehicle model is valid for the TrajectoryType.
 
-        :param vehicle_model: VehicleModel - CommonRoad enum for vehicle models
-        :return: bool - True if the vehicle model is valid for the TrajectoryType
+        :param vehicle_model: CommonRoad enum for vehicle models
+        :return: True if the vehicle model is valid for the TrajectoryType
         """
         return any([
             self.name == 'Input' and vehicle_model in [VehicleModel.KS, VehicleModel.ST, VehicleModel.MB],
@@ -229,7 +281,7 @@ class PlanningProblemSolution:
         self.vehicle_type = vehicle_type
         self._cost_function = cost_function
         self._trajectory = trajectory
-        self._trajectory_type = TrajectoryType.get_trajectory_type(self._trajectory)
+        self._trajectory_type = TrajectoryType.get_trajectory_type(self._trajectory, self.vehicle_model)
 
         self._check_trajectory_supported(self._vehicle_model, self._trajectory_type)
         self._check_cost_supported(self._vehicle_model, self._cost_function)
@@ -241,11 +293,12 @@ class PlanningProblemSolution:
 
         :param vehicle_model: VehicleModel
         :param cost_function: CostFunction
-        :return: bool - True if supported.
+        :return: True if supported.
         """
         supported_costs = SupportedCostFunctions[vehicle_model.name].value
         if cost_function not in supported_costs:
-            raise Exception("Cost function %s isn't supported for %s model!" % (cost_function.name, vehicle_model.name))
+            raise SolutionException("Cost function %s isn't supported for %s model!" % (cost_function.name,
+                                                                                        vehicle_model.name))
         return True
 
     @staticmethod
@@ -255,11 +308,11 @@ class PlanningProblemSolution:
 
         :param vehicle_model: VehicleModel
         :param trajectory_type: TrajectoryType
-        :return: bool - True if valid.
+        :return: True if valid.
         """
         if not trajectory_type.valid_vehicle_model(vehicle_model):
-            raise Exception('Vehicle model %s is not valid for the trajectory type %s!'
-                            % (vehicle_model.name, trajectory_type.name))
+            raise SolutionException('Vehicle model %s is not valid for the trajectory type %s!'
+                                    % (vehicle_model.name, trajectory_type.name))
         return True
 
     @property
@@ -335,28 +388,26 @@ class PlanningProblemSolution:
 
 class Solution:
     """Stores a solution to a CommonRoad benchmark and additional meta data."""
+
     def __init__(self,
-                 scenario_id: str,
-                 commonroad_version: str,
+                 scenario_id: ScenarioID,
                  planning_problem_solutions: List[PlanningProblemSolution],
                  date: datetime = datetime.today(),
                  computation_time: Union[float, None] = None,
                  processor_name: Union[str, None] = None):
         """
         :param scenario_id: Scenario ID of the Solution
-        :param commonroad_version: valid CommonRoad Version (see `:py:data:`~common.file_reader.SUPPORTED_COMMONROAD_VERSIONS`)
         :param planning_problem_solutions: List of PlanningProblemSolution for corresponding
-        to the planning problems of the scenario
+            to the planning problems of the scenario
         :param date: The date solution was produced. Default=datetime.today()
-        :param computation_time: The computation time it took for the Solution. Default=None
+        :param computation_time: The computation time measured in seconds for the Solution. Default=None
         :param processor_name: The processor model used for the Solution. Determined automatically if set to 'auto'.
-        Default=None.
+            Default=None.
         """
-        assert commonroad_version in SUPPORTED_COMMONROAD_VERSIONS
         self.scenario_id = scenario_id
-        self.commonroad_version = commonroad_version
         self.planning_problem_solutions = planning_problem_solutions
         self.date = date
+        self._computation_time = None
         self.computation_time = computation_time
         self.processor_name = processor_name
 
@@ -370,9 +421,9 @@ class Solution:
             VehicleModel = PM
             VehicleType = FORD_ESCORT
             CostFunction = JB1
-            Version = 2018b
+            Version = 2020a
 
-            Benchmark ID = PM1:JB1:TEST:2018b
+            Benchmark ID = PM1:JB1:TEST:2020a
 
         Collaborative Solution Example:
             Scenario ID = TEST
@@ -382,7 +433,7 @@ class Solution:
             2nd VehicleModel = PM
             2nd VehicleType = VW_VANAGON
             2nd CostFunction = SA1
-            Version = 2018b
+            Version = 2020a
 
             Benchmark ID = [PM1,PM3]:[JB1,SA1]:TEST:2020a
 
@@ -392,7 +443,7 @@ class Solution:
         cost_ids = self.cost_ids
         vehicles_str = vehicle_ids[0] if len(vehicle_ids) == 1 else '[%s]' % ','.join(vehicle_ids)
         costs_str = cost_ids[0] if len(cost_ids) == 1 else '[%s]' % ','.join(cost_ids)
-        return '%s:%s:%s:%s' % (vehicles_str, costs_str, self.scenario_id, self.commonroad_version)
+        return '%s:%s:%s:%s' % (vehicles_str, costs_str, str(self.scenario_id), self.scenario_id.scenario_version)
 
     @property
     def vehicle_ids(self) -> List[str]:
@@ -454,11 +505,29 @@ class Solution:
         """
         return [pp_solution.trajectory_type for pp_solution in self.planning_problem_solutions]
 
+    @property
+    def computation_time(self) -> Union[None, float]:
+        """
+        Return the computation time [s] for the trajectory.
+        :return:
+        """
+        return self._computation_time
+
+    @computation_time.setter
+    def computation_time(self, computation_time):
+        if computation_time is not None:
+            assert is_real_number(computation_time), "<Solution> computation_time provided as type {}," \
+                                                     "but expected type float," \
+                                                     "measured in seconds!".format(type(computation_time))
+            assert is_positive(computation_time), "<Solution> computation_time needs to be positive!"\
+                .format(type(computation_time))
+        self._computation_time = computation_time
+
 
 class CommonRoadSolutionReader:
     """Reads solution xml files created with the CommonRoadSolutionWriter"""
+
     # TODO Add parser check
-    # TODO Prepare proper error messages to show to the user.
 
     @classmethod
     def open(cls, filepath: str) -> Solution:
@@ -485,22 +554,30 @@ class CommonRoadSolutionReader:
 
     @classmethod
     def _parse_solution(cls, root_node: et.Element) -> Solution:
-        """ Parses the Solution XML root node. """
+        """ Parses the Solution XML root node. """  # TODO
         benchmark_id, date, computation_time, processor_name = cls._parse_header(root_node)
-        vehicle_ids, cost_ids, scenario_id, version = cls._parse_benchmark_id(benchmark_id)
+        vehicle_ids, cost_ids, scenario_id = cls._parse_benchmark_id(benchmark_id)
         pp_solutions = [cls._parse_planning_problem_solution(vehicle_ids[idx], cost_ids[idx], trajectory_node)
                         for idx, trajectory_node in enumerate(root_node)]
-        return Solution(scenario_id, version, pp_solutions, date, computation_time, processor_name)
+        return Solution(scenario_id, pp_solutions, date, computation_time, processor_name)
 
     @staticmethod
     def _parse_header(root_node: et.Element) -> Tuple[str, Union[None, datetime], Union[None, float], Union[None, str]]:
         """ Parses the header attributes for the given Solution XML root node. """
-        benchmark_id = root_node.get('benchmark_id')
-        date = root_node.attrib.get('date')  # None if not found
-        if date is not None: date = datetime.strptime(date, '%Y-%m-%d')
-        computation_time = root_node.attrib.get('computation_time')
-        if computation_time is not None: computation_time = float(computation_time)
-        processor_name = root_node.attrib.get('processor_name')
+        benchmark_id = root_node.get('benchmark_id', None)
+        if not benchmark_id:
+            SolutionException("Solution xml does not have a benchmark id!")
+
+        date = root_node.attrib.get('date', None)  # None if not found
+        if date is not None:
+            date = datetime.strptime(date, '%Y-%m-%d')
+
+        computation_time = root_node.attrib.get('computation_time', None)
+        if computation_time is not None:
+            computation_time = float(computation_time)
+
+        processor_name = root_node.attrib.get('processor_name', None)
+
         return benchmark_id, date, computation_time, processor_name
 
     @classmethod
@@ -508,14 +585,22 @@ class CommonRoadSolutionReader:
                                          trajectory_node: et.Element) -> PlanningProblemSolution:
         """ Parses PlanningProblemSolution from the given XML node. """
         vehicle_model, vehicle_type = cls._parse_vehicle_id(vehicle_id)
+
+        if cost_id not in [cfunc.name for cfunc in CostFunction]:
+            raise SolutionReaderException("Invalid Cost ID: " + cost_id)
         cost_function = CostFunction[cost_id]
+
         pp_id, trajectory = cls._parse_trajectory(trajectory_node)
         return PlanningProblemSolution(pp_id, vehicle_model, vehicle_type, cost_function, trajectory)
 
     @classmethod
     def _parse_trajectory(cls, trajectory_node: et.Element) -> Tuple[int, Trajectory]:
         """ Parses Trajectory and planning problem id from the given XML node. """
+
+        if trajectory_node.tag not in [ttype.value for ttype in TrajectoryType]:
+            raise SolutionReaderException("Invalid Trajectory Type: " + trajectory_node.tag)
         trajectory_type = TrajectoryType(trajectory_node.tag)
+
         planning_problem_id = int(trajectory_node.get('planningProblem'))
         state_list = [cls._parse_state(trajectory_type.state_type, state_node) for state_node in trajectory_node]
         state_list = sorted(state_list, key=lambda state: state.time_step)
@@ -526,7 +611,7 @@ class CommonRoadSolutionReader:
         """ Parses the sub elements from the given XML node. """
         elem = state_node.find(name)
         if elem is None:
-            raise Exception("Element '%s' couldn't be found in the xml node!" % name)
+            raise SolutionReaderException("Element '%s' couldn't be found in the xml node!" % name)
         value = float(elem.text) if as_float else int(elem.text)
         return value
 
@@ -534,7 +619,7 @@ class CommonRoadSolutionReader:
     def _parse_state(cls, state_type: StateType, state_node: et.Element) -> State:
         """ Parses State from the given XML node. """
         if not state_node.tag == state_type.value:
-            raise Exception("Given xml node is not a '%s' node!" % state_type.value)
+            raise SolutionReaderException("Given xml node is not a '%s' node!" % state_type.value)
 
         state_vals = {}
         for mapping in list(zip(state_type.xml_fields, state_type.fields)):
@@ -544,35 +629,42 @@ class CommonRoadSolutionReader:
                 state_vals[field_name] = np.array([cls._parse_sub_element(state_node, name) for name in xml_name])
             else:
                 state_vals[field_name] = cls._parse_sub_element(state_node, xml_name, as_float=(not xml_name == 'time'))
+
+            if not 'orientation' in state_vals and ('velocity' in state_vals and 'velocity_y' in state_vals):
+                state_vals['orientation'] = math.atan2(state_vals['velocity_y'], state_vals['velocity'])
+
         return State(**state_vals)
 
     @staticmethod
-    def _parse_benchmark_id(benchmark_id: str) -> (List[str], List[str], str, str):
+    def _parse_benchmark_id(benchmark_id: str) -> (List[str], List[str], str):
         """ Parses the given benchmark id string. """
         segments = benchmark_id.replace(' ', '').split(':')
 
         if len(segments) != 4:
-            raise Exception("Invalid Benchmark ID: " + benchmark_id)
+            raise SolutionReaderException("Invalid Benchmark ID: " + benchmark_id)
 
         vehicle_model_ids = re.sub(r'[\[\]]', '', segments[0]).split(',')
         cost_function_ids = re.sub(r'[\[\]]', '', segments[1]).split(',')
-        scenario_id = segments[2]
-        version = segments[3]
+        scenario_id = ScenarioID.from_benchmark_id(segments[2], segments[3])
 
-        return vehicle_model_ids, cost_function_ids, scenario_id, version
+        return vehicle_model_ids, cost_function_ids, scenario_id
 
     @staticmethod
     def _parse_vehicle_id(vehicle_id: str) -> Tuple[VehicleModel, VehicleType]:
         """ Parses the given vehicle id string. """
         if not len(vehicle_id) == 3:
-            raise Exception("Invalid Vehicle ID: " + vehicle_id)
+            raise SolutionReaderException("Invalid Vehicle ID: " + vehicle_id)
+
+        if not vehicle_id[:2] in [vmodel.name for vmodel in VehicleModel]:
+            raise SolutionReaderException("Invalid Vehicle ID: " + vehicle_id)
+
+        if not int(vehicle_id[2]) in [vtype.value for vtype in VehicleType]:
+            raise SolutionReaderException("Invalid Vehicle ID: " + vehicle_id)
 
         return VehicleModel[vehicle_id[:2]], VehicleType(int(vehicle_id[2]))
 
 
 class CommonRoadSolutionWriter:
-    # TODO Add parser check
-    # TODO Prepare proper error messages to show to the user.
 
     def __init__(self, solution: Solution):
         """
@@ -580,12 +672,13 @@ class CommonRoadSolutionWriter:
 
         :param solution: Solution.
         """
+        assert isinstance(solution, Solution)
         self.solution = solution
         self._solution_root = self._serialize_solution(self.solution)
 
     @staticmethod
-    def _get_processor_name() -> Tuple[str, None]:
-        # TODO: compare cpu name with list also used on the web server
+    def _get_processor_name() -> Union[str, None]:
+        # TODO: compare cpu names with the list of cpu names used on web server
 
         delete_from_cpu_name = ['(R)', '(TM)']
 
@@ -602,7 +695,7 @@ class CommonRoadSolutionWriter:
         elif platform.system() == "Darwin":
             os.environ['PATH'] = os.environ['PATH'] + os.pathsep + '/usr/sbin'
             command = "sysctl -n machdep.cpu.brand_string"
-            return subprocess.check_output(command).strip()
+            return str(subprocess.check_output(command, shell=True).strip())
         elif platform.system() == "Linux":
             command = "cat /proc/cpuinfo"
             all_info = str(subprocess.check_output(command, shell=True).strip())
@@ -673,7 +766,10 @@ class CommonRoadSolutionWriter:
         :return: string - Solution XML as string.
         """
         rough_string = et.tostring(self._solution_root, encoding='utf-8')
-        if not pretty: return rough_string
+
+        if not pretty:
+            return rough_string
+
         parsed = minidom.parseString(rough_string)
         return parsed.toprettyxml(indent="  ")
 
@@ -696,8 +792,7 @@ class CommonRoadSolutionWriter:
             raise NotADirectoryError("Directory %s does not exist." % os.path.dirname(fullpath))
 
         if os.path.exists(fullpath) and not overwrite:
-            warnings.warn("File %s already exists. If you want to overwrite it set overwrite=True." % fullpath)
+            raise FileExistsError("File %s already exists. If you want to overwrite it set overwrite=True." % fullpath)
 
-        else:
-            with open(fullpath, 'w') as f:
-                f.write(self.dump(pretty))
+        with open(fullpath, 'w') as f:
+            f.write(self.dump(pretty))
