@@ -14,14 +14,12 @@ from commonroad.scenario.intersection import Intersection
 __author__ = "Christian Pek, Sebastian Maierhofer"
 __copyright__ = "TUM Cyber-Physical Systems Group"
 __credits__ = ["BMW CAR@TUM"]
-__version__ = "2020.1"
-__maintainer__ = "Christian Pek"
+__version__ = "2020.3"
+__maintainer__ = "Sebastian Maierhofer"
 __email__ = "commonroad-i06@in.tum.de"
 __status__ = "released"
 
 
-
-
 class LineMarking(enum.Enum):
     """
     Enum describing different types of line markings, i.e. dashed or solid lines
@@ -30,6 +28,8 @@ class LineMarking(enum.Enum):
     SOLID = 'solid'
     BROAD_DASHED = 'broad_dashed'
     BROAD_SOLID = 'broad_solid'
+    UNKNOWN = 'unknown'
+    NO_MARKING = 'no_marking'
 
 
 class LaneletType(enum.Enum):
@@ -39,16 +39,18 @@ class LaneletType(enum.Enum):
     URBAN = 'urban'
     COUNTRY = 'country'
     HIGHWAY = 'highway'
-    DRIVE_WAY='driveWay'
+    DRIVE_WAY = 'driveWay'
     MAIN_CARRIAGE_WAY = 'mainCarriageWay'
     ACCESS_RAMP = 'accessRamp'
     EXIT_RAMP = 'exitRamp'
     SHOULDER = 'shoulder'
     BUS_LANE = 'busLane'
     BUS_STOP = 'busStop'
-    BIKE_LANE = 'bikeLane'
+    BICYCLE_LANE = 'bicycleLane'
     SIDEWALK = 'sidewalk'
     CROSSWALK = 'crosswalk'
+    INTERSTATE = 'interstate'
+    UNKNOWN = 'unknown'
 
 
 class RoadUser(enum.Enum):
@@ -60,18 +62,19 @@ class RoadUser(enum.Enum):
     TRUCK = 'truck'
     BUS = 'bus'
     PRIORITY_VEHICLE = 'priorityVehicle'
-    MOTORCYCLE = 'motorycle'
+    MOTORCYCLE = 'motorcycle'
     BICYCLE = 'bicycle'
     PEDESTRIAN = 'pedestrian'
     TRAIN = 'train'
+    TAXI = 'taxi'
 
 
 class StopLine:
     """Class which describes the stop line of a lanelet"""
     def __init__(self, start: np.ndarray, end: np.ndarray,
                  line_marking: LineMarking,
-                 traffic_sign_ref: int = None,
-                 traffic_light_ref: int = None):
+                 traffic_sign_ref: Set[int] = None,
+                 traffic_light_ref: Set[int] = None):
         self._start = start
         self._end = end
         self._line_marking = line_marking
@@ -91,13 +94,37 @@ class StopLine:
         return self._line_marking
 
     @property
-    def traffic_sign_ref(self) -> int:
+    def traffic_sign_ref(self) -> Set[int]:
         return self._traffic_sign_ref
 
     @property
-    def traffic_light_ref(self) -> int:
+    def traffic_light_ref(self) -> Set[int]:
         return self._traffic_light_ref
 
+    def translate_rotate(self, translation: np.ndarray, angle: float):
+        """
+        This method translates and rotates a stop line
+
+        :param translation: The translation given as [x_off,y_off] for the x and y translation
+        :param angle: The rotation angle in radian (counter-clockwise defined)
+        """
+
+        assert is_real_number_vector(translation,
+                                     2), '<Lanelet/translate_rotate>: provided translation ' \
+                                         'is not valid! translation = {}'.format(translation)
+        assert is_valid_orientation(
+            angle), '<Lanelet/translate_rotate>: provided angle is not valid! angle = {}'.format(angle)
+
+        # create transformation matrix
+        t_m = commonroad.geometry.transform.translation_rotation_matrix(translation,
+                                                                        angle)
+        line_vertices = np.array([self._start, self._end])
+        # transform center vertices
+        tmp = t_m.dot(np.vstack((line_vertices.transpose(),
+                                 np.ones((1, line_vertices.shape[0])))))
+        tmp = tmp[0:2, :].transpose()
+        self._start, self._end = tmp[0], tmp[1]
+
     def __str__(self):
         return f'StopLine from {self._start} to  {self._end}'
 
@@ -113,12 +140,12 @@ class Lanelet:
                  lanelet_id: int, predecessor=None, successor=None,
                  adjacent_left=None, adjacent_left_same_direction=None,
                  adjacent_right=None, adjacent_right_same_direction=None,
-                 line_marking_left_vertices=None,
-                 line_marking_right_vertices=None,
+                 line_marking_left_vertices=LineMarking.NO_MARKING,
+                 line_marking_right_vertices=LineMarking.NO_MARKING,
                  stop_line=None,
-                 lanelet_type = None,
-                 user_one_way = None,
-                 user_bidirectional = None,
+                 lanelet_type=None,
+                 user_one_way=None,
+                 user_bidirectional=None,
                  traffic_signs=None,
                  traffic_lights=None,
                  ):
@@ -165,12 +192,8 @@ class Lanelet:
             len(left_vertices[0]), len(center_vertices[0]), len(right_vertices[0]))
 
         # Set lane markings
-        self._line_marking_left_vertices = None
-        if line_marking_left_vertices is not None:
-            self.line_marking_left_vertices = line_marking_left_vertices
-        self._line_marking_right_vertices = None
-        if line_marking_right_vertices is not None:
-            self.line_marking_right_vertices = line_marking_right_vertices
+        self._line_marking_left_vertices = line_marking_left_vertices
+        self._line_marking_right_vertices = line_marking_right_vertices
 
         # Set predecessors and successors
         self._predecessor = None
@@ -316,7 +339,7 @@ class Lanelet:
                               LineMarking), '<Lanelet/line_marking_left_vertices>: Provided lane marking type of ' \
                                             'left boundary is not valid! type = {}'.format(
                 type(line_marking_left_vertices))
-            self._line_marking_left_vertices = line_marking_left_vertices
+            self._line_marking_left_vertices = LineMarking.UNKNOWN
         else:
             warnings.warn('<Lanelet/line_marking_left_vertices>: line_marking_left_vertices of lanelet is immutable!')
 
@@ -331,7 +354,7 @@ class Lanelet:
                               LineMarking), '<Lanelet/line_marking_right_vertices>: Provided lane marking type of ' \
                                             'right boundary is not valid! type = {}'.format(
                 type(line_marking_right_vertices))
-            self._line_marking_right_vertices = line_marking_right_vertices
+            self._line_marking_right_vertices = LineMarking.UNKNOWN
         else:
             warnings.warn('<Lanelet/line_marking_right_vertices>: line_marking_right_vertices of lanelet is immutable!')
 
@@ -344,7 +367,7 @@ class Lanelet:
         if self._predecessor is None:
             assert (is_list_of_natural_numbers(predecessor) and len(predecessor) >= 0), '<Lanelet/predecessor>: ' \
                                                                                         'Provided list ' \
-                                                                                        'of predecessors is not valid!' \
+                                                                                        'of predecessors is not valid!'\
                                                                                         'predecessors = {}'.format(
                 predecessor)
             self._predecessor = predecessor
@@ -431,7 +454,7 @@ class Lanelet:
         self._dynamic_obstacles_on_lanelet = obstacle_ids
 
     @property
-    def static_obstacles_on_lanelet(self) -> Set[int]:
+    def static_obstacles_on_lanelet(self) -> Union[None, Set[int]]:
         return self._static_obstacles_on_lanelet
 
     @static_obstacles_on_lanelet.setter
@@ -565,6 +588,10 @@ class Lanelet:
         tmp = tmp[0:2, :]
         self._right_vertices = tmp.transpose()
 
+        # transform the stop line
+        if self._stop_line is not None:
+            self._stop_line.translate_rotate(translation, angle)
+
         # recreate polygon in case it existed
         if self._polygon is not None:
             self._polygon = None
@@ -576,7 +603,8 @@ class Lanelet:
         along the lanelet
 
         :param distance: The distance for the interpolation
-        :return: The interpolated positions on the center/right/left polyline in the form [[x_c,y_c],[x_r,y_r],[x_l,y_l]]
+        :return: The interpolated positions on the center/right/left polyline
+        in the form [[x_c,y_c],[x_r,y_r],[x_l,y_l]]
         """
         assert is_real_number(distance) and np.greater_equal(self.distance[-1],distance)\
                and np.greater_equal(distance, 0), '<Lanelet/interpolate_position>: provided distance is not valid! ' \
@@ -615,8 +643,8 @@ class Lanelet:
         :return: List of bools with True indicating point is enclosed and False otherwise
         """
         assert isinstance(point_list,
-                          ValidTypes.ARRAY), '<Lanelet/contains_points>: provided list of points is not a list! type = {}'.format(
-            type(point_list))
+                          ValidTypes.ARRAY), '<Lanelet/contains_points>: provided list of points is not a list! type ' \
+                                             '= {}'.format(type(point_list))
         assert is_valid_polyline(
             point_list), 'Lanelet/contains_points>: provided list of points is malformed! points = {}'.format(
             point_list)
@@ -698,8 +726,8 @@ class Lanelet:
         """
         Merges obstacle IDs of static obstacles on two lanelets
 
-        :param static_obstacles_on_lanelet1: Obstacle IDs on the first lanelet
-        :param lanelet2: Obstacle IDs on the second lanelet
+        :param obstacles_on_lanelet1: Obstacle IDs on the first lanelet
+        :param obstacles_on_lanelet2: Obstacle IDs on the second lanelet
         :return: Merged obstacle IDs of static obstacles on lanelets
         """
         if len(obstacles_on_lanelet2.items()) > 0:
@@ -725,17 +753,18 @@ class Lanelet:
         assert isinstance(lanelet1, Lanelet), '<Lanelet/merge_lanelets>: lanelet1 is not a valid lanelet object!'
         assert isinstance(lanelet2, Lanelet), '<Lanelet/merge_lanelets>: lanelet1 is not a valid lanelet object!'
         # check connection via successor / predecessor
-        assert lanelet1.lanelet_id in lanelet2.successor or lanelet2.lanelet_id in lanelet1.successor,\
-            '<Lanelet/merge_lanelets>: cannot merge two not connected lanelets! successors of l1 = {}, successors of l2 = {}'.format(
-            lanelet1.successor, lanelet2.successor)
+        assert lanelet1.lanelet_id in lanelet2.successor or lanelet2.lanelet_id in lanelet1.successor \
+            or lanelet1.lanelet_id in lanelet2.predecessor or lanelet2.lanelet_id in lanelet1.predecessor,\
+            '<Lanelet/merge_lanelets>: cannot merge two not connected lanelets! successors of l1 = {}, successors ' \
+            'of l2 = {}'.format(lanelet1.successor, lanelet2.successor)
 
         # check pred and successor
-        if lanelet1.lanelet_id in lanelet2.successor:
-            pred = lanelet2
-            suc = lanelet1
-        else:
+        if lanelet1.lanelet_id in lanelet2.predecessor or lanelet2.lanelet_id in lanelet1.successor:
             pred = lanelet1
             suc = lanelet2
+        else:
+            pred = lanelet2
+            suc = lanelet1
 
         # build new merged lanelet (remove first node of successor if both lanes are connected)
         # check connectedness
@@ -767,7 +796,8 @@ class Lanelet:
                                                         max_length: float = 150.0, lanelet_type: LaneletType = None) \
             -> Tuple[List['Lanelet'], List[List[int]]]:
         """
-        Computes all reachable lanelets starting from a provided lanelet and merges them to a single lanelet for each route.
+        Computes all reachable lanelets starting from a provided lanelet
+        and merges them to a single lanelet for each route.
 
         :param lanelet: The lanelet to start from
         :param network: The network which contains all lanelets
@@ -776,11 +806,13 @@ class Lanelet:
         :return: List of merged lanelets, Lists of lanelet ids of which each merged lanelet consists
         """
         assert isinstance(lanelet, Lanelet), '<Lanelet>: provided lanelet is not a valid Lanelet!'
-        assert isinstance(network, LaneletNetwork), '<Lanelet>: provided lanelet network is not a valid lanelet network!'
-        assert network.find_lanelet_by_id(lanelet.lanelet_id) is not None, '<Lanelet>: lanelet not contained in network!'
+        assert isinstance(network, LaneletNetwork), '<Lanelet>: provided lanelet network is not a ' \
+                                                    'valid lanelet network!'
+        assert network.find_lanelet_by_id(lanelet.lanelet_id) is not None, '<Lanelet>: lanelet not ' \
+                                                                           'contained in network!'
 
         if lanelet.successor is None or len(lanelet.successor) == 0:
-            return [], []
+            return [lanelet], [[lanelet.lanelet_id]]
 
         # Create Graph from network
         Net = nx.DiGraph() 
@@ -863,6 +895,14 @@ class Lanelet:
         """
         self.traffic_signs.add(traffic_sign_id)
 
+    def add_traffic_light_to_lanelet(self, traffic_light_id: int):
+        """
+        Adds a traffic light ID to lanelet
+
+        :param traffic_light_id: traffic light ID to add
+        """
+        self.traffic_lights.add(traffic_light_id)
+
     def dynamic_obstacle_by_time_step(self, time_step) -> Set[int]:
         """
         Returns all dynamic obstacles on lanelet at specific time step
@@ -885,10 +925,10 @@ class LaneletNetwork:
     """
 
     def __init__(self):
-        self._lanelets: Dict[int,Lanelet] = {}
-        self._intersections: Dict = {}
-        self._traffic_signs: Dict = {}
-        self._traffic_lights: Dict = {}
+        self._lanelets: Dict[int, Lanelet] = {}
+        self._intersections: Dict[int, Intersection] = {}
+        self._traffic_signs: Dict[int, TrafficSign] = {}
+        self._traffic_lights: Dict[int, TrafficLight] = {}
 
     @property
     def lanelets(self) -> List[Lanelet]:
@@ -929,8 +969,8 @@ class LaneletNetwork:
         """
         assert isinstance(lanelets, list) and all(isinstance(l, Lanelet) for l in
                                                   lanelets), '<LaneletNetwork/create_from_lanelet_list>:' \
-                                                             'Provided list of lanelets is not valid! lanelets = {}'.format(
-            lanelets)
+                                                             'Provided list of lanelets is not valid! ' \
+                                                             'lanelets = {}'.format(lanelets)
 
         # create lanelet network
         lanelet_network = cls()
@@ -970,15 +1010,11 @@ class LaneletNetwork:
             l._adj_left_same_direction = None if l._adj_left_same_direction is None \
                                                  or l.adj_left not in exisiting_ids \
                 else l._adj_left_same_direction
-            if prev != l._adj_left_same_direction:
-                print(prev,l._adj_left_same_direction)
             l._adj_right = None if l._adj_right is None or l._adj_right not in exisiting_ids else l._adj_right
             prev = copy.deepcopy(l._adj_right_same_direction)
             l._adj_right_same_direction = None if l._adj_right_same_direction is None \
                                                   or l.adj_right not in exisiting_ids \
                 else l._adj_right_same_direction
-            if prev != l._adj_right_same_direction:
-                print(prev,l._adj_right_same_direction)
 
     def find_lanelet_by_id(self, lanelet_id: int) -> Lanelet:
         """
@@ -1018,6 +1054,18 @@ class LaneletNetwork:
 
         return self._traffic_lights[traffic_light_id] if traffic_light_id in self._traffic_lights else None
 
+    def find_intersection_by_id(self, intersection_id: int) -> Intersection:
+        """
+        Finds a intersection for a given intersection_id
+
+        :param intersection_id: The id of the intersection to find
+        :return: The intersection object if the id exists and None otherwise
+        """
+        assert is_natural_number(intersection_id), '<LaneletNetwork/find_intersection_by_id>: ' \
+                                                   'provided id is not valid! id = {}'.format(intersection_id)
+
+        return self._intersections[intersection_id] if intersection_id in self._intersections else None
+
     def add_lanelet(self, lanelet: Lanelet):
         """
         Adds a lanelet to the LaneletNetwork
@@ -1051,7 +1099,8 @@ class LaneletNetwork:
 
         # check if traffic already exists in network and warn user
         if traffic_sign.traffic_sign_id in self._traffic_signs.keys():
-            warnings.warn('Traffic sign already exists in network! No changes are made.')
+            warnings.warn('Traffic sign with ID {} already exists in network! '
+                          'No changes are made.'.format(traffic_sign.traffic_sign_id))
             return False
         else:
             self._traffic_signs[traffic_sign.traffic_sign_id] = traffic_sign
@@ -1085,7 +1134,7 @@ class LaneletNetwork:
             for lanelet_id in lanelet_ids:
                 lanelet = self.find_lanelet_by_id(lanelet_id)
                 if lanelet is not None:
-                    lanelet.add_traffic_sign_to_lanelet(traffic_light.traffic_light_id)
+                    lanelet.add_traffic_light_to_lanelet(traffic_light.traffic_light_id)
                 else:
                     warnings.warn('Traffic light cannot be referenced to lanelet because the lanelet does not exist.')
             return True
@@ -1133,14 +1182,18 @@ class LaneletNetwork:
         """
 
         assert is_real_number_vector(translation,
-                                     2), '<LaneletNetwork/translate_rotate>: provided translation is not valid! translation = {}'.format(
-            translation)
+                                     2), '<LaneletNetwork/translate_rotate>: provided translation is not valid! ' \
+                                         'translation = {}'.format(translation)
         assert is_valid_orientation(
             angle), '<LaneletNetwork/translate_rotate>: provided angle is not valid! angle = {}'.format(angle)
 
         # rotate each lanelet
         for lanelet in self._lanelets.values():
             lanelet.translate_rotate(translation, angle)
+        for traffic_sign in self._traffic_signs.values():
+            traffic_sign.translate_rotate(translation, angle)
+        for traffic_light in self._traffic_lights.values():
+            traffic_light.translate_rotate(translation, angle)
 
     def find_lanelet_by_position(self, point_list: List[np.ndarray]) -> List[List[int]]:
         """
@@ -1150,8 +1203,8 @@ class LaneletNetwork:
         :return: A list of lanelet ids. If the position could not be matched to a lanelet, an empty list is returned
         """
         assert isinstance(point_list,
-                          ValidTypes.LISTS), '<Lanelet/contains_points>: provided list of points is not a list! type = {}'.format(
-            type(point_list))
+                          ValidTypes.LISTS), '<Lanelet/contains_points>: provided list of points is not a list! ' \
+                                             'type = {}'.format(type(point_list))
         # assert is_valid_polyline(
         #     point_list), 'Lanelet/contains_points>: provided list of points is malformed! points = {}'.format(
         #     point_list)
